/* * Copyright 2021 Philippe Even, Université de Lorraine, IUT de St Dié.
 *     2023 Noémy Chevalier Contributor
 *     2023 Louise Berrada Contributor
 *     2023 Piquerez Corentin Contributor
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package shukan;


/**
 * List of week queues of activities.
 * It features a queue for each week.
 * Modified by Chevalier Noemy, Berrada Louise and Piquerez Corentin
 */
public class ShukanSchedule
{
  /** Bin capacity */
  private int capacity = 0;
  /** Sequences of bins */
  private ShukanQueue[] binSet = null;


  /** Create a list of queues */
  public ShukanSchedule (int n, int capa)
  {
    capacity = capa;
    binSet = new ShukanQueue[n];
    for (int i = 0; i < n; i++) binSet[i] = new ShukanQueue (capa);
  }

  /** Kill the nth queue */
  public void kill (int n)
  {
    binSet[n].kill ();
  }

  /** Return the contents of the nth queue */
  public int[] contents (int n)
  {
    return (binSet[n].contents ());
  }

  /** Return the contents of the sluice */
  public int[] contents ()
  {
    int[] sched = new int[100];
    int act = 0;
    for (int i = 0; i < binSet.length; i++)
    {
      int tl = binSet[i].length ();
      for (int j = 0; j < tl; j++)
      {
        if (act == sched.length)
        {
          int[] tmp = new int[sched.length + 100];
          for (int k = 0; k < sched.length; k++) tmp[k] = sched[k];
          sched = tmp;
        }
        sched[act++] = i;
      }
    }
    int[] tmp = new int[act];
    for (int i = 0; i < act; i++) tmp[i] = sched[i];
    return (tmp);
  }

  /** Clear the queues */
  public void clear ()
  {
    for (int i = 0; i < binSet.length; i++) binSet[i].clear ();
  }

  /** delete one element of one queue and update all the queues */
  public void delete(int index, int week, int nb_week){
    binSet[week].delete(index);
    for (int i =index; i<binSet[week].length(); i++){
      binSet[week].removeOne(i);
    }
    for(int i =week+1; i<nb_week; i++) binSet[i].removeAllOne();
  }

  /** Add a token to the nth queue */
  public void add (int n, int token)
  {
    binSet[n].pushRight (token);
  }

  /** Push a token leftwards the nth queue */
  public int left (int n)
  {
    if (binSet[n].empty ()) return (n);
    int left = n - 1;
    while (left >= 0 && binSet[left].killed ()) left--;
    if (left >= 0 && ! binSet[left].full ())
    {
      binSet[left].pushRight (binSet[n].popLeft ());
      return (left);
    }
    return (n);
  }

  /** Push a token rightwards the nth queue */
  public int right (int n)
  {
    if (binSet[n].empty ()) return (n);
    int right = n + 1;
    while (right < binSet.length && binSet[right].killed ()) right++;
    if (right < binSet.length && ! binSet[right].full ())
    {
      binSet[right].pushLeft (binSet[n].popRight ());
      return (right);
    }
    return (n);
  }

  /** Push tokens leftwards inside the queue */
  public int left (int n1, int n2)
  {
    while (binSet[n1].killed () && n1 < n2) n1 ++;
    while (binSet[n2].killed () && n2 > n1) n2 --;
    for (int i = n1; i < n2; i ++) left (i + 1);
    return (n1);
  }

  /** Push tokens rightwards inside the queue */
  public int right (int n1, int n2)
  {
    while (binSet[n1].killed () && n1 < n2) n1 ++;
    while (binSet[n2].killed () && n2 > n1) n2 --;
    for (int i = n2; i > n1; i --) right (i - 1);
    return (n1);
  }

  /** Returns the week and activity at given position. */
  public int select (int pos)
  {
    int selBin = pos / capacity;
    int selAct = pos % capacity;
    if (binSet[selBin].length () <= selAct)
    {
      boolean looking = true;
      // Looking frontwards
      int sbin = selBin;
      while (looking && sbin >= 0)
      {
        if (binSet[sbin].length () > 0)
        {
          looking = false;
          selBin = sbin;
          selAct = binSet[sbin].length () - 1;
        }
        sbin --;
      }
      if (looking)
      {
        // Looking backwards
        sbin = selBin + 1;
        while (looking && sbin < binSet.length)
        {
          if (binSet[sbin].length () > 0)
          {
            looking = false;
            selBin = sbin;
            selAct = 0;
          }
          sbin ++;
        }
      }
      if (looking)
      {
        selBin = sbin;
        selAct = 0;
        return (-1);
      }
    }
    return (selBin * capacity + selAct);
  }

  /** Returns the activity number of the given slot (week + activity) */
  public int identify (int slot)
  {
    int activ = 0;
    for (int i = 0; i < binSet.length; i++)
      if (slot < capacity)
      {
        activ += slot;
        i = binSet.length;
      }
      else
      {
        activ += binSet[i].length ();
        slot -= capacity;
      }
    return (activ);
  }
}
