/* * Copyright 2021 Philippe Even, Université de Lorraine, IUT de St Dié.
 *     2023 Noémy Chevalier Contributor
 *     2023 Louise Berrada Contributor
 *     2023 Piquerez Corentin Contributor
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package shukan;
import java.util.Calendar;
import java.text.DecimalFormat;


/**
 * Semester arrangement
 * Modified by Chevalier Noemy, Berrada Louise and Piquerez Corentin
 */
public class ShukanSemester
{
  /** Month names */
  private static final String[] MONTH_FULL_NAMES = {
    "janvier", "fevrier", "mars", "avril", "mai", "juin",
    "juillet", "aout", "septembre", "octobre", "novembre", "decembre"};
  /** Month short names */
  private static final String[] MONTH_ACRONYMS = {
    "janv.", "fev.", "mars", "avr.", "mai", "juin",
    "juil.", "aout", "sept.", "oct.", "nov.", "dec."};

  /** Year of cursus start */
  private int startYear;
  /** Names of each week */
  private String[] weekNames;
  /** Maximal duration of longest week (in hours) */
  private int standardWeekDuration;
  /** Maximal duration of each week (in hours) */
  private int[] weekDurations;
  /** Number of first cursus week */
  private int firstWeek;
  /** Number of the last week in the cursus year (could be inferred) */
  private int lastWeekOfYear;


  /** Creates a semester from a directory name */
  public ShukanSemester (int year, int first, int length, int duration)
  {
    startYear = year;
    Calendar cal = Calendar.getInstance ();
    cal.set (Calendar.YEAR, year);
    lastWeekOfYear = cal.getActualMaximum (Calendar.WEEK_OF_YEAR);
    firstWeek = first;

    weekNames = new String[length];
    standardWeekDuration = duration;
    weekDurations = new int[length];
    int weekNo = first;
    DecimalFormat df = new DecimalFormat ("00");
    for (int i = 0; i < length; i++)
    {
      cal.set (Calendar.WEEK_OF_YEAR, weekNo);
      cal.set (Calendar.DAY_OF_WEEK, 0);
      weekNames[i] = df.format (cal.get (Calendar.DAY_OF_MONTH))
                     + " " + MONTH_ACRONYMS[cal.get (Calendar.MONTH)];
      weekDurations[i] = duration;
      if (++weekNo > lastWeekOfYear) weekNo = 1;
    }
  }

  /** Sets a week duration */
  public boolean setDuration (int week, int duration)
  {
    if (week > lastWeekOfYear) return false;
    week -= firstWeek;
    if (week < 0) week += lastWeekOfYear;
    if (week >= weekDurations.length) return false;
    weekDurations[week] = duration;
    return true;
  }

  /** Returns the number of weeks */
  public int size ()
  {
    return (weekDurations.length);
  }

  /** Returns the standard week duration */
  public int standardWeekDuration ()
  {
    return (standardWeekDuration);
  }

  /** Returns the week durations */
  public int[] weekDurations ()
  {
    return (weekDurations);
  }

  /** Returns a copy of the week durations */
  public int[] copyWeekDurations ()
  {
    int[] wd = new int[weekDurations.length];
    for (int i = 0; i < weekDurations.length; i++) wd[i] = weekDurations[i];
    return (wd);
  }

  /** Returns the semesterLength */
  public int length ()
  {
    return (weekDurations.length);
  }

  /** Returns the week numbers */
  public int[] weekNumbers ()
  {
    int[] nb = new int[weekDurations.length];
    int num = firstWeek;
    for (int i = 0; i < nb.length; i++)
    {
      nb[i] = num++;
      if (num > lastWeekOfYear) num = 1;
    }
    return (nb);
  }

  /** Returns a week number */
  public int weekNumber (int weekIndex)
  {
    int wk = firstWeek + weekIndex;
    if (wk > lastWeekOfYear) wk -= lastWeekOfYear;
    return (wk);
  }

  /** Returns the start week number */
  public int startWeekNumber ()
  {
    return (firstWeek);
  }

  /** Returns the last week of year number */
  public int lastWeekOfYearNumber ()
  {
    return (lastWeekOfYear);
  }

  /** Returns the index of a year week */
  public int weekIndex (int week)
  {
    if (week < firstWeek) return (week + lastWeekOfYear - firstWeek);
    else return (week - firstWeek);
  }

  /** Converts semester to string */
  public String toString ()
  {
    String s = new String ();
    for (int i = 0; i < weekNames.length; i++) s += weekNames[i] + "\n";
    return s;
  }

  /** Returns the present date in text format */
  public String date ()
  {
    Calendar kal = Calendar.getInstance ();
    DecimalFormat format = new DecimalFormat ("00");
    return ((kal.get (Calendar.YEAR) % 100)
            + format.format (kal.get (Calendar.MONTH) + 1)
            + format.format (kal.get (Calendar.DAY_OF_MONTH)));
  }

  /** Returns the year of cursus start */
  public int yearOfFirstWeek ()
  {
    return (startYear);
  }
}
