/* * Copyright 2021 Philippe Even, Université de Lorraine, IUT de St Dié.
 *     2023 Noémy Chevalier Contributor
 *     2023 Louise Berrada Contributor
 *     2023 Piquerez Corentin Contributor
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package shukan;
import javax.swing.*;
import javax.swing.plaf.ColorUIResource;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


/**
 * Graphical view for Shukan planner.
 * Modified by Chevalier Noemy, Berrada Louise and Piquerez Corentin
 */
public class ShukanView extends JPanel
{
  /** Version number */
  public static final long serialVersionUID = 2L;

  /** Font type */
  //private final int FONT = GLUT.BITMAP_HELVETICA_12;

  /** Max window width */
  private final int MAX_WIDTH = 1610;
  /** Max window height */
  private final int MAX_HEIGHT = 960;
  /** Security margin at the window border for other applications */
  private final int SECURITY_MARGIN = 70;

  /** Maximal height of a text line */
  private final int TEXT_MAX_HEIGHT = 15;
  /** Minimal height of a text line */
  private final int TEXT_MIN_HEIGHT = 10;
  /** Standard height of a text line */
  private final int STD_TEXT_HEIGHT = 13;

  /** Standard week column width */
  private final static int STD_WEEK_WIDTH = 30;
  /** Standard activity symbol width */
  private final static int STD_ACTIV_WIDTH = 6;
  /** Standard number of modules */
  private final static int STD_MODULES_NUMBER = 14;
  /** Standard number of weeks */
  private final static int STD_WEEKS_NUMBER = 20;

  /** Standard application width */
  private final static int STD_APPLI_WIDTH =
                           STD_WEEK_WIDTH * (STD_WEEKS_NUMBER + 2);
  /** Standard application height */
  private final int STD_APPLI_HEIGHT =
                    STD_TEXT_HEIGHT * 3 * (STD_MODULES_NUMBER + 1);

  /** Week column actual width */
  private int weekWidth = STD_WEEK_WIDTH;
  /** Standard activity symbol actual width */
  private int activWidth = STD_ACTIV_WIDTH;
  /** Number of weeks to display */
  private int nbWeeks = STD_WEEKS_NUMBER;
  /** Number of modules to display */
  private int nbModules = STD_MODULES_NUMBER;
  /** Height of a text line */
  private int textHeight = STD_TEXT_HEIGHT;
  /** Application actual width */
  private int appliWidth = STD_APPLI_WIDTH;
  /** Application actual height */
  private int appliHeight = STD_APPLI_HEIGHT;

  /** Height of the font characters */
  private int fontHeight = 10;
  /** Font metrics features */
  private FontMetrics fontMetrics;


  /** Background color */
  private Color BACK_COLOR = new ColorUIResource(226, 226, 226);
  /** Background color */
  private Color BACK_COLOR2 = Color.LIGHT_GRAY;
  /** Grid color */
  private final Color GRID_COLOR = Color.BLACK;
  /** Free module weeks color */
  private  Color HOLLY_COLOR = new Color (1.f, 0.8f, 0.7f);
  /** Inactive parts of weeks color */
  private  Color INACTIVE_COLOR = new Color (0.5f, 0.5f, 0.5f);
  /** Frame borders color */
  private final Color ACTIVE_COLOR = new Color (0.6f, 0.6f, 0.9f);
  /** Selected activity color */
  private final Color REACTIVE_COLOR = new Color (0.7f, 0.9f, 0.5f);
  /** Frame borders color */
  private final Color FRAME_COLOR = Color.BLACK;
  /** Text color */
  private Color TEXT_COLOR = Color.BLACK;
  /** Selected text color */
  private final Color SELECTED_TEXT_COLOR = Color.WHITE;
  /** Unselectable text color */
  private final Color UNSEL_TEXT_COLOR = new Color (0.6f, 0.6f, 0.6f);
  /** Selected slot central color */
  private final Color SELECTION_COLOR = new Color (0.6f, 0.6f, 0.6f);

  /** File manager */
  private ShukanIO myIO = null;
  /** Displayed data */
  private ShukanData data = null;

  /** Creates the shukan viewer.
   */
  public ShukanView (ShukanData data, ShukanIO myIO)
  {
    this.data = data;
    this.myIO = myIO;  
  }

  /** Saves Shukan data in a file.
   */
  public void saveData ()
  {
    myIO.save ();
  }

  /** Sets the initial application size.
   */
  public void restrictSize (int width, int height)
  {
    adaptSize (width, height);
  }

  /** Returns the display area width.
   */
  public int displayWidth ()
  {
    return (appliWidth);
  }

  /** Returns the display area height.
   */
  public int displayHeight ()
  {
    return (appliHeight);
  }

  /** Draws Shukan view.
    */
  public void paintComponent (Graphics g)
  {
    int w = getWidth ();
    int h = getHeight ();
    adaptSize (w, h);

    Graphics2D g2 = (Graphics2D) g.create ();
    g2.setFont (new Font ("Helvetica", 0, 12));
    fontMetrics = g2.getFontMetrics ();
    fontHeight = fontMetrics.getAscent ();
    g2.setColor(BACK_COLOR2);
    drawBox (g2, appliWidth - w, appliHeight - h, w, h);
    int i;
    for (i = 0; i <= nbModules + 2; i++)
    {
      int y = appliHeight - (i + 1) * 3 * textHeight + 2*textHeight;


      if (i % 2 == 1)
      {
        g2.setColor (BACK_COLOR);
        drawBox (g2,0, y, w, 3 * textHeight);
      }
      else
      {
        g2.setColor (BACK_COLOR2);
        drawBox (g2, 0, y, w, 3 * textHeight);
      }
    }

    displayCalendar (g2);

  }



  /** Detects a zone from window coords.
   */
  public boolean select (int x, int y, boolean zone)
  {
    // Out of the active part of the window
    if (y < textHeight || y > appliHeight - 2 * textHeight) return (false);
    if (x < 0 || x > appliWidth) return (false);

    // Module detection
    y -= textHeight;
    int selMod = y / (3 * textHeight);

    // Module selection
    if (x < 2 * weekWidth) return (data.activate (selMod));
    // Activity selection
    x -= 2 * weekWidth;
    return (data.activate (selMod, (x / weekWidth), zone));
  }


  /** Detects an activity from window coords.
   */
  public boolean selectActivity (int x, int y, boolean zone)
  {
    // Out of the active part of the window
    if (y < textHeight || y > appliHeight - 2 * textHeight) return (false);

    // Module detection
    y -= textHeight;
    int selMod = y / (3 * textHeight);

    // Module selection
    if (x < 2 * weekWidth) return (data.activateAct (selMod));
    // Activity selection
    x -= 2 * weekWidth;
    int selw = x / weekWidth;
    int seln = (x - (selw * weekWidth)) / activWidth;
    return (data.select (selMod,
                         selw * ShukanModule.MAX_ACTIV_PER_WEEK + seln,
                         zone));
  }


  /** Updates the application size.
   */
  private void adaptSize (int width, int height)
  {
    if (data != null)
    {
      nbWeeks = data.semesterSize ();
      nbModules = data.numberOfModules ();

      // Height
      textHeight = (int) (height / (3.0f * (nbModules + 2)));
      appliHeight = textHeight * 3 * (nbModules + 2);

      // Width
      activWidth = width / (ShukanModule.MAX_ACTIV_PER_WEEK * (nbWeeks + 2));
      weekWidth = activWidth * ShukanModule.MAX_ACTIV_PER_WEEK;
      appliWidth = weekWidth * (nbWeeks + 2);
    }
    else
    {
      appliWidth = STD_APPLI_WIDTH;
      appliHeight = STD_APPLI_HEIGHT;
    }
  }

  /** Displays the calendar grid background.
   */
  private void displayCalendar (Graphics2D g2)
  {
    // Selection
    int selMod = data.selectedModule ();
    if (selMod != -1)
    {
      int selWeek = data.selectedWeek ();
      if (selWeek != -1)
      {
        int selLength = data.selectedLength ();
        g2.setColor (ACTIVE_COLOR);
        int x = (selWeek + 2) * weekWidth;
        int y = appliHeight - textHeight - (selMod + 1) * textHeight * 3;
        drawBox (g2, x, y, weekWidth * selLength, textHeight * 3);
      }
      else
      {
        int sa = data.selectedActivity ();
        if (sa != -1)
        {
          g2.setColor (REACTIVE_COLOR);
          drawBox (g2, sa * activWidth + 2 * weekWidth,
                   appliHeight - textHeight - (selMod + 1) * textHeight * 3,
                   activWidth * data.selectedLength (), textHeight * 3);
        }
      }
    }

    // Modules holly periods
    int[] durations = data.weekDurations ();
    g2.setColor (HOLLY_COLOR);
    for (int j = 0; j < nbModules; j++)
    {
      int[] aw = data.activeWeeks (j);
      for (int i = 0; i < nbWeeks; i++)
        if (aw[i] == 0 && durations[i] != 0)
        {
          drawBox (g2, (2 + i) * weekWidth,
                       appliHeight + (2 - 3 * (j + 2)) * textHeight,
                       weekWidth, 3 * textHeight);
        }
    }

    // Holydays
    int wd = data.standardWeekDuration ();
    g2.setColor (INACTIVE_COLOR);
    for (int i = 0; i < nbWeeks; i++)
      if (durations[i] != wd)
      {
        int start = (int) ((durations[i] * weekWidth) / (float) wd);
        drawBox (g2, (2 + i) * weekWidth + start, 5* textHeight,
                     weekWidth - start, appliHeight - 3 * textHeight);
      }

    g2.setColor (GRID_COLOR);
    // Columns
    for (int i = 2; i <= nbWeeks + 1; i++)
      drawVLine (g2, i * weekWidth, 0, appliHeight);
    // Lines
    for (int i = 0; i <= nbModules +1 ; i++)
      drawHLine (g2, 0, (2 + 3 * i) * textHeight, appliWidth);

    // Texts : semester and load name
    drawText (g2, 0, appliHeight - textHeight, 2 * weekWidth, textHeight, data.cursusName ());
    drawText (g2, 0, textHeight - 1, 2 * weekWidth, textHeight, data.loadName ());


    //Text : student name 
    drawText (g2, 0, 3 * textHeight, 2* weekWidth, textHeight, "Etudiant");
    for (int i = 0; i < nbWeeks; i++)
    {
      int[] weekNumbers = data.weekNumbers ();
      int[] hour = data.weekDurations ();
      drawText (g2, (i + 2) * weekWidth, 61,
                            weekWidth, textHeight, "" + data.studentLoad(i));
      drawText (g2, (2 + i) * weekWidth, 0,
                            weekWidth, 106, "/" + hour[i]);
    }


    // Texts : weeks
    int[] weekNumbers = data.weekNumbers ();
    for (int i = 0; i < nbWeeks; i++)
    {
      drawText (g2, (i + 2) * weekWidth, appliHeight - textHeight,
                    weekWidth, textHeight, "S" + weekNumbers[i]);
      drawText (g2, (i + 2) * weekWidth, textHeight,
                    weekWidth, textHeight, "" + data.computeLoad (i));
      drawText (g2, (2 + i) * weekWidth, 0,
                    weekWidth, textHeight, "/" + durations[i]);
    }
    // Texts : modules
    String[] modNames = data.moduleNames ();
    for (int i = 0; i < nbModules; i++)
      drawText (g2, 0, appliHeight - ((i + 1) * 3 + 1) * textHeight,
                    2 * weekWidth, 3 * textHeight, modNames[i]);
    // Texts : activities
    for (int i = 0; i < nbModules; i++)
    {
      int[] act = data.activities (i);
      boolean[] up = data.onLeft (i);
      boolean[] down = data.onRight (i);
      int numact = 0;
      int y = appliHeight - ((i + 1) * 3 + 1) * textHeight;
      for (int j = 0; j < nbWeeks; j++)
      {
        int[] sched = data.scheduleInWeek (i, j);
        for (int k = 0; k < sched.length; k++)
        {
          Font boldFont = new Font(g2.getFont().getFontName(), Font.BOLD, g2.getFont().getSize());
          g2.setFont(boldFont);
          int ly = y;
          if (up[numact]) ly += textHeight * 2;
          else if (! down[numact]) ly += textHeight;
          numact++;
          drawText (g2, (j + 2) * weekWidth + k * activWidth, ly,
                        activWidth, textHeight,
                        ShukanModule.DISPLAY[act[sched[k]]]);
        }
      }
    }
  }


  /** Draws a horizontal line.
   */
  private void drawHLine (Graphics2D g2, int x, int y, int l)
  {
    g2.drawLine (x, appliHeight - y, x + l, appliHeight - y);
  }

  /** Draws a vertical line.
   */
  private void drawVLine (Graphics2D g2, int x, int y, int l)
  {
    g2.drawLine (x, appliHeight - y, x, appliHeight - y - l);
  }

  /** Draws a rectangular box.
   */
  private void drawBox (Graphics2D g2, int posx, int posy,
                                       int width, int height)
  {
    g2.fillRect (posx, appliHeight - posy - height, width, height);
  }

  /** Draws a centered text in the given area.
   */
  private void drawText (Graphics2D g2, float posx, float posy,
                         float width, float height, String text)
  {
    g2.setColor(TEXT_COLOR);
    g2.drawString (text,
                   (int) (posx + (width - fontMetrics.stringWidth (text)) / 2),
                   appliHeight - (int) (posy + (height - fontHeight) / 2));
  }

  /** Draws a centered paragraph in the given area.
   */
  private void drawText (Graphics2D g2, float posx, float posy,
                         float width, float height, String[] text)
  {
    if (text == null) return;
    posy += (height + (text.length - 1) * fontHeight) / 2;
    for (int i = 0; i < text.length; i ++)
    {
      drawText (g2, posx, posy, width, fontHeight, text[i]);
      posy -= fontHeight;
    }
  }
  /** creates a new Window with the specification of a selected week and module */
  public void SpecWeekModuleWindow(){
    int num_module = data.selectedModule();
    int num_cursus = data.cursusNumber();
    int num_semaine = data.selectedWeek()+data.startWeekNumber();
    int[] week_activities = data.scheduleInWeek(num_module, data.selectedWeek());
    // affiche la fenetre si il y a cours pendant cette periode
    if(week_activities.length>0)
    {
      JFrame spec = new JFrame(data.cursusName(num_cursus) + " - Semaine" + (num_semaine) + " - " + data.moduleNames(num_cursus)[num_module]);


      int[] activities = data.activities(num_module);
      int nb_cours = week_activities.length;
      boolean rajout_prof = false;

      JPanel cours = new JPanel(new GridLayout(nb_cours*3, 1));
      JLabel[] noms_cours = new JLabel[nb_cours];
      JLabel[] duree = new JLabel[nb_cours];
      JLabel[] profs = new JLabel[nb_cours];
      JLabel[] salles = new JLabel[nb_cours];
      String nom_profs;
      for (int i =0; i<nb_cours; i++)
      {
        String[] teachers = data.moduleTeachers(num_cursus, num_module, activities[week_activities[i]]);
        if (teachers.length>0){
          nom_profs = teachers[0];
          for (int j = 1; j < teachers.length; j++) {
            rajout_prof = true;
            for (int k = 0; k < j; k++) {
              if (teachers[j] == teachers[k]) rajout_prof = false;
            }
            if (rajout_prof) nom_profs += " / " + teachers[j];
          }
        }
        else nom_profs = data.subtitle(num_module);

        noms_cours[i] =  new JLabel("Cours : "+data.activityMainName(activities[week_activities[i]]));
        noms_cours[i].setFont(new Font(noms_cours[i].getFont().getFamily(), noms_cours[i].getFont().getStyle()+Font.BOLD, noms_cours[i].getFont().getSize()+10));
        duree[i] = new JLabel("   Durée : "+data.activityDuration(activities[week_activities[i]])+"h");
        profs[i] = new JLabel("   Professeur : "+ nom_profs);
        salles[i] = new JLabel("   Salle : "+ data.activityRoom(activities[week_activities[i]]));
        cours.add(noms_cours[i]);
        cours.add(duree[i]);
        cours.add(profs[i]);
        cours.add(salles[i]);
      }

      spec.add(cours);
      spec.setSize(new Dimension(350, 100*nb_cours));
      spec.setVisible(true);
    }
  }

   /** Change the background color */
  public void setBackgroundColor(Color c, Color c1){
    BACK_COLOR = c;
    BACK_COLOR2 = c1;
    repaint();
  }

  /** Change the text and the hollydays colors*/
  public void setForeground(Color c, Color c1, Color c2){
    TEXT_COLOR = c;
    HOLLY_COLOR = c1;
    INACTIVE_COLOR = c2;
    repaint();
  }

  
  /** Creates a new activity with window */
  public void newActivity(){
    JFrame f = new JFrame("Informations concernant le nouveau cours");
    JPanel global_mod = new JPanel();
    JPanel global_act = new JPanel();
    JLabel mod = new JLabel("A quelle matiere voulez vous rajouter un cours ?");
    JLabel act = new JLabel("Quel cours voulez vous ajouter? ");
    JComboBox mod_info = new JComboBox(data.moduleNames(data.cursusNumber()));
    JComboBox act_info = new JComboBox(ShukanModule.TYPES);
    JButton valider = new JButton("Valider");

    valider.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        int num_mod = mod_info.getSelectedIndex();
        int num_act = act_info.getSelectedIndex();
        int cursus = data.cursusNumber();
        int mod_start = data.moduleStart(cursus, num_mod);
        if (mod_start!=0) mod_start-=data.startWeekNumber();
        int nb_act = data.scheduleInWeek(num_mod, mod_start).length;

        data.addNewActivity(cursus, num_mod, nb_act, ShukanModule.TYPES[num_act]);
        data.scheduleActivity(cursus, num_mod, nb_act, mod_start);
        data.setOnMiddle(cursus, num_mod, nb_act, true);
        int[] weeks = data.schedule(num_mod);
        data.unscheduleActivities(cursus, num_mod);
        data.setSchedule(cursus, num_mod, weeks);

        repaint();
        f.dispose();
      }
    });

    global_mod.add(mod);
    global_mod.add(mod_info);
    global_act.add(act);
    global_act.add(act_info);
    f.add(global_mod, BorderLayout.NORTH);
    f.add(global_act, BorderLayout.CENTER);
    f.add(valider, BorderLayout.SOUTH);
    f.setVisible(true);
    f.setSize(new Dimension(400, 150));
  }

  /** Creates a new activity with window */
  public void deleteActivity(){
    JFrame f = new JFrame("Informations concernant le cours à supprimer");
    String[] s = new String[data.semesterSize()];
    for (int i =0; i<data.semesterSize(); i++)
      s[i] = (data.startWeekNumber()+i)+"";
    JPanel global_mod = new JPanel();
    JPanel global_act = new JPanel();
    JPanel global_sem = new JPanel();
    JPanel global = new JPanel(new GridLayout(3,0));
    JLabel info = new JLabel("Vous allez supprimer le dernier cours de la semaine choisie");
    JLabel mod = new JLabel("A quelle matiere voulez vous supprimer un cours ?");
    JLabel sem = new JLabel("Pendant quelle semaine le cours se déroule t-il ?");
    JComboBox mod_info = new JComboBox(data.moduleNames(data.cursusNumber()));
    JComboBox sem_info = new JComboBox(s);
    JButton valider = new JButton("Valider");

    valider.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        int num_mod = mod_info.getSelectedIndex();
        int num_sem = sem_info.getSelectedIndex();
        int cursus = data.cursusNumber();
        int nb_act = data.scheduleInWeek(num_mod, num_sem).length;

        if (nb_act>0){
          data.deleteActivity(cursus, num_mod, num_sem, nb_act-1);
          data.unscheduleActivity(cursus, num_mod, nb_act, num_sem, data.semesterSize());
          repaint();
        }
        f.dispose();
      }
    });

    global_mod.add(mod);
    global_mod.add(mod_info);
    global_sem.add(sem);
    global_sem.add(sem_info);
    global.add(global_mod);
    global.add(global_act);
    global.add(global_sem);
    f.add(info, BorderLayout.NORTH);
    f.add(global, BorderLayout.CENTER);
    f.add(valider, BorderLayout.SOUTH);
    f.setVisible(true);
    f.setSize(new Dimension(400, 200));
  }
}
